@@ -5,7 +5,7 @@ namespace select {
 
 void SelectTraits::set_options(std::vector<std::string> options) { this->options_ = std::move(options); }
 
-std::vector<std::string> SelectTraits::get_options() const { return this->options_; }
+const std::vector<std::string> &SelectTraits::get_options() const { return this->options_; }
 
 }  // namespace select
 }  // namespace esphome
