@@ -6,6 +6,22 @@ namespace text_sensor {
 
 static const char *const TAG = "text_sensor";
 
+void log_text_sensor(const char *tag, const char *prefix, const char *type, TextSensor *obj) {
+  if (obj == nullptr) {
+    return;
+  }
+
+  ESP_LOGCONFIG(tag, "%s%s '%s'", prefix, type, obj->get_name().c_str());
+
+  if (!obj->get_device_class_ref().empty()) {
+    ESP_LOGCONFIG(tag, "%s  Device Class: '%s'", prefix, obj->get_device_class_ref().c_str());
+  }
+
+  if (!obj->get_icon_ref().empty()) {
+    ESP_LOGCONFIG(tag, "%s  Icon: '%s'", prefix, obj->get_icon_ref().c_str());
+  }
+}
+
 void TextSensor::publish_state(const std::string &state) {
   this->raw_state = state;
   if (this->raw_callback_) {
