@@ -215,12 +215,37 @@ void Tuya::handle_command_(uint8_t command, uint8_t version, const uint8_t *buff
         this->send_empty_command_(TuyaCommandType::DATAPOINT_QUERY);
       }
       break;
-    case TuyaCommandType::WIFI_RESET:
-      ESP_LOGE(TAG, "WIFI_RESET is not handled");
-      break;
     case TuyaCommandType::WIFI_SELECT:
-      ESP_LOGE(TAG, "WIFI_SELECT is not handled");
+    case TuyaCommandType::WIFI_RESET: {
+      const bool is_select = (len >= 1);
+      // Send WIFI_SELECT ACK
+      TuyaCommand ack;
+      ack.cmd = is_select ? TuyaCommandType::WIFI_SELECT : TuyaCommandType::WIFI_RESET;
+      ack.payload.clear();
+      this->send_command_(ack);
+      // Establish pairing mode for correct first WIFI_STATE byte, EZ (0x00) default
+      uint8_t first = 0x00;
+      const char *mode_str = "EZ";
+      if (is_select && buffer[0] == 0x01) {
+        first = 0x01;
+        mode_str = "AP";
+      }
+      // Send WIFI_STATE response, MCU exits pairing mode
+      TuyaCommand st;
+      st.cmd = TuyaCommandType::WIFI_STATE;
+      st.payload.resize(1);
+      st.payload[0] = first;
+      this->send_command_(st);
+      st.payload[0] = 0x02;
+      this->send_command_(st);
+      st.payload[0] = 0x03;
+      this->send_command_(st);
+      st.payload[0] = 0x04;
+      this->send_command_(st);
+      ESP_LOGI(TAG, "%s received (%s), replied with WIFI_STATE confirming connection established",
+               is_select ? "WIFI_SELECT" : "WIFI_RESET", mode_str);
       break;
+    }
     case TuyaCommandType::DATAPOINT_DELIVER:
       break;
     case TuyaCommandType::DATAPOINT_REPORT_ASYNC:
