@@ -18,6 +18,12 @@ class UARTDevice {
 
   void write_byte(uint8_t data) { this->parent_->write_byte(data); }
 
+  void set_rx_full_threshold(size_t rx_full_threshold) { this->parent_->set_rx_full_threshold(rx_full_threshold); }
+  void set_rx_full_threshold_ms(size_t time) { this->parent_->set_rx_full_threshold_ms(time); }
+  size_t get_rx_full_threshold() { return this->parent_->get_rx_full_threshold(); }
+  void set_rx_timeout(size_t rx_timeout) { this->parent_->set_rx_timeout(rx_timeout); }
+  size_t get_rx_timeout() { return this->parent_->get_rx_timeout(); }
+
   void write_array(const uint8_t *data, size_t len) { this->parent_->write_array(data, len); }
   void write_array(const std::vector<uint8_t> &data) { this->parent_->write_array(data); }
   template<size_t N> void write_array(const std::array<uint8_t, N> &data) {
